@@ -2,13 +2,13 @@ package Future::Q;
 use strict;
 use warnings;
 use Future 0.22;
-use base "Future";
+use parent "Future";
 use Devel::GlobalDestruction;
 use Scalar::Util qw(refaddr blessed weaken);
 use Carp;
 use Try::Tiny ();
 
-our $VERSION = '0.080';
+our $VERSION = '0.090';
 
 our @CARP_NOT = qw(Try::Tiny Future);
 
@@ -115,7 +115,7 @@ sub then {
         }
         $next_future->resolve($return_future);
     });
-    if($next_future->is_pending) {
+    if($next_future->is_pending && $self->is_pending) {
         weaken(my $invo_future = $self);
         $next_future->on_cancel(sub {
             if(defined($invo_future) && $invo_future->is_pending) {
@@ -165,10 +165,12 @@ sub resolve {
             $self->fulfill($base_future->get);
         }
     });
-    weaken(my $weak_base = $base_future);
-    $self->on_cancel(sub {
-        $weak_base->cancel() if defined($weak_base) && !$weak_base->is_ready;
-    });
+    if(!$base_future->is_ready) {
+        weaken(my $weak_base = $base_future);
+        $self->on_cancel(sub {
+            $weak_base->cancel() if defined($weak_base) && !$weak_base->is_ready;
+        });
+    }
     return $self;
 }
 
@@ -204,6 +206,46 @@ foreach my $method (qw(wait_all wait_any needs_all needs_any)) {
     };
 }
 
+sub finally {
+    my ($self, $callback) = @_;
+    my $class = ref($self);
+    $self->_q_set_failure_handled();
+    if(!defined($callback) || ref($callback) ne "CODE") {
+        return $class->new->reject("Callback for finally() must be a code-ref");
+    }
+    my $next_future = $self->new;
+    $self->on_ready(sub {
+        my ($invo_future) = @_;
+        if($invo_future->is_cancelled) {
+            $next_future->cancel if $next_future->is_pending;
+            return;
+        }
+        my $returned_future = $class->try($callback);
+        $returned_future->on_ready(sub {
+            my ($returned_future) = @_;
+            if(!$returned_future->is_cancelled && $returned_future->failure) {
+                $next_future->resolve($returned_future);
+            }else {
+                $next_future->resolve($invo_future);
+            }
+        });
+        if(!$returned_future->is_ready) {
+            weaken(my $weak_returned = $returned_future);
+            $next_future->on_cancel(sub {
+                $weak_returned->cancel if defined($weak_returned) && !$weak_returned->is_ready;
+            });
+        }
+    });
+    if(!$self->is_ready) {
+        weaken(my $weak_invo = $self);
+        $next_future->on_cancel(sub {
+            $weak_invo->cancel if defined($weak_invo) && !$weak_invo->is_ready;
+        
+        });
+    }
+    return $next_future;
+}
+
 1;
 
 __END__
@@ -338,11 +380,15 @@ L<Future::Q> thinks failures of the following futures are "handled".
 
 =item *
 
-Futures that C<then()> or C<catch()> method has been called on.
+Futures that C<then()>, C<catch()> or C<finally()> method has been called on.
 
 =item *
 
-Futures returned by C<$on_fulfilled> or C<$on_rejected> callbacks for C<then()> method.
+Futures returned by C<$on_fulfilled> or C<$on_rejected> callbacks for C<then()> or C<catch()> method.
+
+=item *
+
+Futures returned by C<$callback> for C<finally()> method.
 
 =item *
 
@@ -502,12 +548,16 @@ When C<$future> is fulfilled and C<$on_fulfilled> is provided,
 C<$on_fulfilled> is executed.
 In this case C<$next_future> represents the result of C<$on_fulfilled> callback (see below).
 
+    @returned_values = $on_fulfilled->(@values)
+
 =item *
 
 When C<$future> is rejected and C<$on_rejected> is provided,
 C<$on_rejected> is executed.
 In this case C<$next_future> represents the result of C<$on_rejected> callback (see below).
 
+    @returned_values = $on_rejected->($exception, @detail)
+
 =item *
 
 In the above two cases where C<$on_fulfilled> or C<$on_rejected> callback is executed,
@@ -547,14 +597,98 @@ is pending, the C<$returned_future> is cancelled when C<$next_future> is cancell
 
 You should not call C<fulfill()>, C<reject()>, C<resolve()> etc on C<$next_future>.
 
-Since C<then()> method passes the C<$future>'s state to C<$next_future>,
+Because C<then()> method passes the C<$future>'s failure to C<$on_rejected> callback or C<$next_future>,
 C<$future>'s failure becomes "handled", i.e., L<Future::Q> won't warn you
 if C<$future> is rejected and DESTROYed.
 
+
 =head2 $next_future = $future->catch([$on_rejected])
 
 Alias of C<< $future->then(undef, $on_rejected) >>.
 
+=head2 $next_future = $future->finally($callback)
+
+Registers a callback function that is executed when C<$future> is either fulfilled or rejected.
+This callback is analogous to "finally" block of try-catch-finally statements found in Java etc.
+
+It returns a new L<Future::Q> object (C<$next_future>) that keeps the result of the operation.
+
+The mandatory argument, C<$callback>, is a subroutine reference.
+It is executed with no arguments when C<$future> is either fulfilled or rejected.
+
+    @returned_values = $callback->()
+
+If C<$callback> finishes successfully, C<$next_future> has the same state and values as C<$future>.
+That is, if C<$future> is fulfilled C<$next_future> becomes fulfilled with the same values,
+and if C<$future> is rejected C<$next_future> becomes rejected with the same failure.
+In this case the return values of C<$callback> are discarded.
+
+If C<$callback> fails, C<$next_future> is rejected with the failure thrown by C<$callback>.
+In this case the values of C<$future> are discarded.
+
+In detail, the state of C<$next_future> is determined by the following rules.
+
+=over
+
+=item *
+
+When C<$future> is pending, C<$next_future> is pending.
+
+=item *
+
+When C<$future> is cancelled, C<$callback> is not executed, and C<$next_future> becomes cancelled.
+
+=item *
+
+When C<$future> is fulfilled or rejected, C<$callback> is executed with no arguments.
+C<$next_future>'s state depends on the result of C<$callback>.
+
+=item *
+
+If the C<$callback> returns a single L<Future> (call it C<$returned_future>),
+C<$next_future> waits for C<$returned_future> to become non-pending state.
+
+=over
+
+=item *
+
+When C<$returned_future> is pending, C<$next_future> is pending.
+
+=item *
+
+When C<$returned_future> is fulfilled or cancelled, C<$next_future> has the same state and values as B<< C<$future> >>.
+In this case, values of C<$returned_future> are discarded.
+
+=item *
+
+When C<$returned_future> is rejected, C<$next_future> is rejected with C<$returned_future>'s failure.
+
+=back
+
+=item *
+
+If the C<$callback> throws an exception, C<$next_future> is rejected with that exception.
+The exception is never rethrown to the upper stacks.
+
+=item *
+
+Otherwise, C<$next_future> has the same state and values as B<< C<$future> >>.
+Values returned from C<$callback> are discarded.
+
+=back
+
+You can call C<cancel()> method on C<$next_future>.
+If C<$future> is pending, it is cancelled when C<$next_future> is cancelled.
+If C<$callback> returns a single L<Future> and the C<$returned_future> is pending,
+the C<$returned_future> is cancelled when C<$next_future> is cancelled.
+
+You should not call C<fulfill()>, C<reject()>, C<resolve()> etc on C<$next_future>.
+
+Because C<finally()> method passes the C<$future>'s failure to C<$next_future>,
+C<$future>'s failure becomes "handled", i.e., L<Future::Q> won't warn you
+if C<$future> is rejected and DESTROYed.
+
+
 =head2 $future = $future->fulfill(@result)
 
 Fulfills the pending C<$future> with the values C<@result>.
@@ -655,7 +789,26 @@ This method is inherited from L<Future>.
     });
     $f->reject("a", "b", "c");
 
+=head2 finally()
 
+    use Future::Q;
+    
+    my $handle;
+    
+    ## Suppose Some::Resource->open() returns a handle to a resource (like
+    ## database) wrapped in a Future
+    
+    Some::Resource->open()->then(sub {
+        $handle = shift;
+        return $handle->read_data(); ## Read data asynchronously
+    })->then(sub {
+        my $data = shift;
+        print "Got data: $data\n";
+    })->finally(sub {
+        ## Ensure closing the resource handle. This callback is called
+        ## even when open() or read_data() fails.
+        $handle->close() if $handle; 
+    });
 
 =head1 DIFFERENCE FROM Q
 
@@ -700,10 +853,9 @@ Some of them worth noting are listed below.
 L<Future> already has C<fail()> method for a completely different meaning.
 Use C<catch()> method instead.
 
-=item promise.progress(), deferred.notify(), promise.finally(), promise.fin()
+=item promise.progress(), deferred.notify()
 
-Progress handlers and "finally" callbacks are interesting features,
-but they are not supported in this version of L<Future::Q>.
+Progress handlers are not supported in this version of L<Future::Q>.
 
 =item promise.done()
 
@@ -720,6 +872,12 @@ Use C<< Future::Q->fcall() >>.
 
 Use C<< Future::Q->needs_all() >> and C<< Future::Q->wait_all() >> methods inherited from the original L<Future> class.
 
+=item promise.inspect()
+
+Use predicate methods C<is_pending()>, C<is_fulfilled()>, C<is_rejected()> and C<is_cancelled()>.
+To obtain values from a fulfilled L<Future>, use C<get()> method.
+To obtain the reason of the failure from a rejected L<Future>, use C<failure()> method.
+
 =item Q()
 
 Use C<< Future::Q->wrap() >> method inherited from the original L<Future> class.
@@ -772,6 +930,12 @@ Because Q is also an implementation of Promises/A+, L<Promises> and Q (and L<Fut
 Another port of Q (implementation of Promises/A+) in Perl.
 It depends on L<AnyEvent>.
 
+=item L<AnyEvent::Promise>
+
+A simple Promise used with L<AnyEvent> condition variables. Apparently it has nothing to do with Promises/A+.
+
+
+     [AnyEvent::Promise]
                 [Future] -\
                            +-- [Future::Q]
     [Promises/A+] -- [Q] -/
